#define WIN32_LEAN_AND_MEAN
#define VC_EXTRA_LEAN
#include <windows.h>

#if defined(UNICODE)
#   define NAME_SUFFIX  TEXT("W")
#else
#   define NAME_SUFFIX  TEXT("A")
#endif

LONG RunScreenSaver(const bool Setup);
void ShowError(const DWORD Error = GetLastError());

#if defined(UNICODE)
extern "C" void __stdcall hkWinMainCRTStartupW()
#else // _UNICODE
extern "C" void __stdcall hkWinMainCRTStartupA()
#endif // _UNICODE
{
    // What should we do?
    LPCTSTR pCommandLine = GetCommandLine();
    for (;
         (*pCommandLine != '\0') && (*pCommandLine != ' ');
         ++pCommandLine);
    if ( *pCommandLine == ' ' )
        while (*++pCommandLine == ' ');
    if ( ( lstrcmpi(pCommandLine, TEXT("enable")) == 0 ) ||
         ( lstrcmpi(pCommandLine, TEXT("on")) == 0 ) )
    {
        if (!SystemParametersInfo( SPI_SETSCREENSAVEACTIVE, TRUE, 0, SPIF_SENDWININICHANGE ))
            ShowError();
        MessageBeep(MB_OK);
    }
    else if ( ( lstrcmpi(pCommandLine, TEXT("disable")) == 0 ) ||
              ( lstrcmpi(pCommandLine, TEXT("off")) == 0 ) )
    {
        if (!SystemParametersInfo( SPI_SETSCREENSAVEACTIVE, FALSE, 0, SPIF_SENDWININICHANGE ))
            ShowError();
        MessageBeep(MB_OK);
    }
    else if ( ( lstrcmpi(pCommandLine, TEXT("status")) == 0 ) ||
              ( lstrcmpi(pCommandLine, TEXT("info")) == 0 ) )
    {
        BOOL IsActive;
        if (!SystemParametersInfo( SPI_GETSCREENSAVEACTIVE, 0, &IsActive, 0 ))
            ShowError();
        MessageBox( NULL,
                    IsActive ?
                        TEXT("Screen saver is active") :
                        TEXT("Screen saver is *NOT* active"),
                    TEXT("hkSSControl v1.0"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST );
        ExitProcess( IsActive ? 100 : 99 );
        MessageBeep(MB_OK);
    }
    else if ( lstrcmpi(pCommandLine, TEXT("now")) == 0 )
    {
        ExitProcess( RunScreenSaver(false) );
    }
    else if ( ( lstrcmpi(pCommandLine, TEXT("setup")) == 0 ) ||
              ( lstrcmpi(pCommandLine, TEXT("config")) == 0 ) )
    {
        ExitProcess( RunScreenSaver(true) );
    }
    else
    {
        MessageBox( NULL,
                    TEXT("Usage: hkSSControl") NAME_SUFFIX TEXT(" <Enable|Disable|Status|Now|Setup>\n")
                    TEXT("where\n")
                    TEXT("\tEnable\t-- Enable the screen saver\n")
                    TEXT("\tDisable\t-- Disable the screen saver\n")
                    TEXT("\tStatus\t-- Display the screen saver's status\n")
                    TEXT("\tNow\t-- Active the screen saver\n")
                    TEXT("\tSetup\t-- Display the screen saer's setup dialog (if any)\n")
                    TEXT("Or, if you prefer: hkSSControl") NAME_SUFFIX TEXT(" <On|Off|Info|Activate|Config>\n")
                    TEXT("\n")
                    TEXT("Copyright \251 1999 by Howard Kapustein, All Rights Reserved\n")
                    TEXT("This program is Freeware. If you like it, drop me a note.\n")
                    TEXT("\n")
                    TEXT("Email: hsk0@voicenet.com\n")
                    TEXT("Web:   http://www.voicenet.com/~hsk0"),
                    TEXT("hkSSControl v1.0"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST );
        ExitProcess( 1 );
    }
    ExitProcess( 0 );
}

LONG RunScreenSaver(const bool Setup)
{
    // ScreenSaver 'EXE' stored at:
    //
    //  Key:    HKCU\Control Panel\Desktop
    //  Name:   SCRNSAVE.EXE
    //  Value:  REG_SZ -- fully-qualified filename
    HKEY hKey;
    LONG RegRC = RegOpenKeyEx( HKEY_CURRENT_USER,
                               TEXT("Control Panel\\Desktop"),
                               0,
                               KEY_READ,
                               &hKey );
    if ( RegRC != ERROR_SUCCESS )
    {
        MessageBox( NULL,
                    TEXT("Error accessing HKCU\\Control Panel\\Desktop"),
                    TEXT("hkSSControl v1.0"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST );
        return RegRC;
    }
    TCHAR  ScreenSaverCommandLine[ MAX_PATH + 1 + 3 ];
    ULONG ValueType;
    ULONG ValueLength = sizeof(ScreenSaverCommandLine);
    RegRC = RegQueryValueEx( hKey,
                             TEXT("SCRNSAVE.EXE"),
                             0,
                             &ValueType,
                             reinterpret_cast<BYTE*>( ScreenSaverCommandLine ),
                             &ValueLength );
    RegCloseKey( hKey );
    if ( RegRC != ERROR_SUCCESS )
    {
        MessageBox( NULL,
                    TEXT("Error accessing registry key SCRNSAVE.EXE=... (under HKCU\\Control Panel\\Desktop)"),
                    TEXT("hkSSControl v1.0"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST );
        return RegRC;
    }
    if ( ! Setup )
        lstrcat( ScreenSaverCommandLine, TEXT(" /S") );

    STARTUPINFO StartupInfo;
    memset( &StartupInfo, 0, sizeof(StartupInfo) );
    StartupInfo.cb = sizeof(StartupInfo);
    PROCESS_INFORMATION ProcessInfo;
    if ( CreateProcess( 0,
                        ScreenSaverCommandLine,
                        0,
                        0,
                        FALSE,
                        0,
                        0,
                        0,
                        &StartupInfo,
                        &ProcessInfo ) )
    {
        if ( ProcessInfo.hProcess != NULL )
            CloseHandle(ProcessInfo.hProcess);
        if ( ProcessInfo.hThread != NULL )
            CloseHandle(ProcessInfo.hThread);
    }

    // Say boo
    MessageBeep(MB_OK);

    // Done
    return ERROR_SUCCESS;
}

void ShowError(const DWORD Error)
{
    LPTSTR pMessage = 0;
    FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
                       FORMAT_MESSAGE_FROM_SYSTEM |
                       FORMAT_MESSAGE_IGNORE_INSERTS,
                   NULL,
                   Error,
                   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                   static_cast<LPTSTR>(pMessage),
                   0,
                   NULL );
    MessageBox( NULL,
                pMessage,
                TEXT("hkSSControl v1.0"),
                MB_OK | MB_ICONERROR | MB_SETFOREGROUND | MB_TOPMOST );
    LocalFree( pMessage );
    ExitProcess( Error );
}
